SimpleSelfRebuff:_rev("$Revision: 43191 $")

-------------------------------------------------------------------------------
-- Externals
-------------------------------------------------------------------------------

local SimpleSelfRebuff = SimpleSelfRebuff
local L = AceLibrary("AceLocale-2.2"):new("SimpleSelfRebuff")

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local bindingButton
local TEH_BUTTON_NAME = "SimpleSelfRebuff_MouseWheelBindingButton"
local boundBuff

local updateEvents = {
	'SimpleSelfRebuff_PendingCastChanged',
	'SimpleSelfRebuff_MonitoringEnabled',
	'PLAYER_REGEN_ENABLED',
	'SPELL_UPDATE_COOLDOWN',
	'BAG_UPDATE_COOLDOWN'
}

-------------------------------------------------------------------------------
-- Module declaration
-------------------------------------------------------------------------------

local MouseWheelBinding = SimpleSelfRebuff:NewModule('MouseWheelBinding')

MouseWheelBinding.optionName = L["Mouse wheel bindings"]

function MouseWheelBinding:OnModuleEnable()
	self:RegisterBucketEvent(updateEvents, 0.1, 'UpdateBinding')
	
	self:RegisterEvent('PLAYER_REGEN_DISABLED', 'SetBinding', false)
	self:RegisterEvent('SimpleSelfRebuff_MonitoringDisabled', 'SetBinding', false)
	
	self:UpdateBinding()
end

function MouseWheelBinding:OnModuleDisable()
	self:SetBinding(false)
end

function MouseWheelBinding:UpdateBinding()
	local buff = self.core:GetPendingCast()
	if buff and buff:IsInCooldown() then
		self:ScheduleEvent('SimpleSelfRebuff_RecheckCooldown', self.UpdateBinding, 0.5, self)
		self:SetBinding(false)
	else
		self:SetBinding(buff)
	end
end

function MouseWheelBinding:SetBinding(buff)

	if buff and not buff.mountFriendly and (IsMounted() or IsStealthed()) then
		buff = nil
	end
	
	if buff then
		
		if InCombatLockdown() or buff == boundBuff then
			return
		end

		self.core:Debug('Binding mousewheel')
		
		if not bindingButton then
			bindingButton = CreateFrame("Button", TEH_BUTTON_NAME, UIParent, "SecureActionButtonTemplate")
			bindingButton:RegisterForClicks("LeftButtonUp")
			bindingButton:SetPoint("CENTER",0,0)
			bindingButton:Hide()
		end
		
		-- Reset attributes
		bindingButton:SetAttribute('type', nil)
		bindingButton:SetAttribute('spell', nil)			
		bindingButton:SetAttribute('unit', nil)			
		bindingButton:SetAttribute('target-slot', nil)			
		
		-- Setup attributes depending on source and target
		buff:SetupSecureButton(bindingButton)
		
		-- Binding
		SetOverrideBindingClick(bindingButton, true, "MOUSEWHEELUP", TEH_BUTTON_NAME, "LeftButton")
		SetOverrideBindingClick(bindingButton, true, "MOUSEWHEELDOWN", TEH_BUTTON_NAME, "LeftButton")	
		
		boundBuff = buff
		
	elseif boundBuff then
		
		if InCombatLockdown() then
			self:ScheduleLeaveCombatAction('SetBinding', false)
		else		
			self.core:Debug('Unbinding mousewheel')
			ClearOverrideBindings(bindingButton)
			boundBuff = nil
		end
		
	end

end
